@@ -7,7 +7,7 @@
 # it under the terms of version 2 of the GNU General Public License as
 # published by the Free Software Foundation.
 #
-# $Id: __init__.py 3877 2006-09-25 01:28:35Z piman $
+# $Id: __init__.py 3888 2006-10-03 05:58:26Z piman $
 #
 
 """Mutagen aims to be an all purpose tagging library.
@@ -21,7 +21,7 @@ depending on tag or format. They may also be entirely different objects
 for certain keys, again depending on format.
 """
 
-version = (1, 7, 1)
+version = (1, 8, -1)
 version_string = ".".join(map(str, version))
 
 import warnings
@@ -167,8 +167,10 @@ def File(filename, options=None):
         from mutagen.wavpack import WavPack
         from mutagen.m4a import M4A
         from mutagen.musepack import Musepack
+        from mutagen.monkeysaudio import MonkeysAudio
         options = [MP3, TrueAudio, OggTheora, OggSpeex, OggVorbis, OggFLAC,
-                   FLAC, APEv2File, M4A, ID3FileType, WavPack, Musepack]
+                   FLAC, APEv2File, M4A, ID3FileType, WavPack, Musepack,
+                   MonkeysAudio]
 
     if not options:
         return None
