@@ -12,6 +12,7 @@ from mutagen.wavpack import WavPack
 from mutagen.trueaudio import TrueAudio
 from mutagen.m4a import M4A
 from mutagen.musepack import Musepack
+from mutagen.monkeysaudio import MonkeysAudio
 try: from os.path import devnull
 except ImportError: devnull = "/dev/null"
 
@@ -109,6 +110,12 @@ class TFile(TestCase):
         self.failUnless(isinstance(
             File(os.path.join("tests", "data", "sv5_header.mpc")), Musepack))
 
+    def test_monkeysaudio(self):
+        self.failUnless(isinstance(
+            File(os.path.join("tests", "data", "mac-399.ape")), MonkeysAudio))
+        self.failUnless(isinstance(
+            File(os.path.join("tests", "data", "mac-396.ape")), MonkeysAudio))
+
     def test_apev2(self):
         self.failUnless(isinstance(
             File(os.path.join("tests", "data", "oldtag.apev2")), APEv2File))
@@ -126,3 +133,19 @@ class TFile(TestCase):
             File(os.path.join("tests", "data", "has-tags.m4a")), M4A))
 
 add(TFile)
+
+class TMutagen(TestCase):
+    uses_mmap = False
+
+    def test_safe_version(self):
+        from mutagen import version_string
+        path = os.path.join("../../releases/mutagen-%s" % version_string)
+        self.failIf(os.path.isdir(path), "mutagen.version is out of date.")
+
+    def test_not_prerelease(self):
+        from mutagen import version
+        from os.path import dirname, basename, abspath
+        trunk = basename(dirname(dirname(dirname(abspath(__file__)))))
+        if trunk != "trunk":
+            self.failIf(version[-1] < 0, "mutagen.version is a prerelease.")
+add(TMutagen)
