@@ -2,8 +2,12 @@
 # it doesn't match the other Mutagen test conventions/quality.
 
 import os
-from tests import add
-from unittest import TestCase
+import shutil
+
+from tempfile import mkstemp
+
+from tests import TestCase, add
+
 import mutagen.apev2
 
 from mutagen.apev2 import APEv2File, APEv2
@@ -14,7 +18,7 @@ OLD = os.path.join(DIR, "data", "oldtag.apev2")
 BROKEN = os.path.join(DIR, "data", "brokentag.apev2")
 LYRICS2 = os.path.join(DIR, "data", "apev2-lyricsv2.mp3")
 
-class APEWriter(TestCase):
+class TAPEWriter(TestCase):
     offset = 0
 
     def setUp(self):
@@ -96,83 +100,28 @@ class APEWriter(TestCase):
         self.failUnlessEqual(
             os.path.getsize(filename), len("tag garbage") * 1000)
 
+    def test_case_preservation(self):
+        mutagen.apev2.delete(SAMPLE + ".justtag")
+        tag = mutagen.apev2.APEv2(SAMPLE + ".new")
+        tag["FoObaR"] = "Quux"
+        tag.save()
+        tag = mutagen.apev2.APEv2(SAMPLE + ".new")
+        self.failUnless("FoObaR" in tag.keys())
+        self.failIf("foobar" in tag.keys())
+
     def tearDown(self):
         os.unlink(SAMPLE + ".new")
         os.unlink(BROKEN + ".new")
         os.unlink(SAMPLE + ".justtag")
         os.unlink(SAMPLE + ".tag_at_start")
 
-class APEReader(TestCase):
-    uses_mmap = False
+add(TAPEWriter)
 
-    def setUp(self):
-        self.tag = mutagen.apev2.APEv2(OLD)
-
-    def test_invalid(self):
-        self.failUnlessRaises(IOError, mutagen.apev2.APEv2, "dne")
-
-    def test_no_tag(self):
-        self.failUnlessRaises(IOError, mutagen.apev2.APEv2,
-                              os.path.join("tests", "data", "empty.mp3"))
-
-    def test_cases(self):
-        self.failUnlessEqual(self.tag["artist"], self.tag["ARTIST"])
-        self.failUnless("artist" in self.tag)
-        self.failUnless("artisT" in self.tag)
-
-    def test_keys(self):
-        self.failUnless("track" in self.tag.keys())
-        self.failUnless("Track" in self.tag.keys())
-        self.failUnless("AnArtist" in self.tag.values())
-
-        self.failUnlessEqual(
-            self.tag.items(), zip(self.tag.keys(), self.tag.values()))
-
-    def test_dictlike(self):
-        self.failUnless(self.tag.get("track"))
-        self.failUnless(self.tag.get("Track"))
-
-    def test_del(self):
-        s = self.tag["artist"]
-        del(self.tag["artist"])
-        self.failIf("artist" in self.tag)
-        self.failUnlessRaises(KeyError, self.tag.__getitem__, "artist")
-        self.tag["Artist"] = s
-        self.failUnlessEqual(self.tag["artist"], "AnArtist")
-
-    def test_values(self):
-        self.failUnlessEqual(self.tag["artist"], self.tag["artist"])
-        self.failUnless(self.tag["artist"] < self.tag["title"])
-        self.failUnlessEqual(self.tag["artist"], "AnArtist")
-        self.failUnlessEqual(self.tag["title"], "Some Music")
-        self.failUnlessEqual(self.tag["album"], "A test case")
-        self.failUnlessEqual("07", self.tag["track"])
-
-        self.failIfEqual(self.tag["album"], "A test Case")
-
-    def test_pprint(self):
-        self.failUnless(self.tag.pprint())
-
-class APEv2ThenID3v1Reader(APEReader):
-    uses_mmap = False
-
-    def setUp(self):
-        import shutil
-        shutil.copy(OLD, OLD + ".new")
-        f = file(OLD + ".new", "ab+")
-        f.write("TAG" + "\x00" * 125)
-        f.close()
-        self.tag = mutagen.apev2.APEv2(OLD + ".new")
-
-    def tearDown(self):
-        os.unlink(OLD + ".new")
-add(APEv2ThenID3v1Reader)
-
-class APEv2ThenID3v1Writer(APEWriter):
+class TAPEv2ThenID3v1Writer(TAPEWriter):
     offset = 128
 
     def setUp(self):
-        super(APEv2ThenID3v1Writer, self).setUp()
+        super(TAPEv2ThenID3v1Writer, self).setUp()
         f = file(SAMPLE + ".new", "ab+")
         f.write("TAG" + "\x00" * 125)
         f.close()
@@ -186,125 +135,16 @@ class APEv2ThenID3v1Writer(APEWriter):
     def test_tag_at_start_write(self):
         pass
 
-add(APEv2ThenID3v1Writer)
-
-class APEv2WithLyrics2(TestCase):
-    uses_mmap = False
-
-    def setUp(self):
-        self.tag = mutagen.apev2.APEv2(LYRICS2)
-
-    def test_values(self):
-        self.failUnlessEqual(self.tag["MP3GAIN_MINMAX"], "000,179")
-        self.failUnlessEqual(self.tag["REPLAYGAIN_TRACK_GAIN"], "-4.080000 dB")
-        self.failUnlessEqual(self.tag["REPLAYGAIN_TRACK_PEAK"], "1.008101")
-
-add(APEv2WithLyrics2)
-
-class APEKeyTest(TestCase):
-    uses_mmap = False
-
-    from mutagen.apev2 import APEKey
-
-    def test_eq(self):
-        self.failUnlessEqual(self.APEKey("foo"), "foo")
-        self.failUnlessEqual("foo", self.APEKey("foo"))
-        self.failUnlessEqual(self.APEKey("foo"), u"foo")
-        self.failUnlessEqual(u"foo", self.APEKey("foo"))
-
-        self.failUnlessEqual(self.APEKey("Bar"), "baR")
-        self.failUnlessEqual(u"baR", self.APEKey("Bar"))
-
-    def test_hash(self):
-        self.failUnlessEqual(hash("foo"), hash(self.APEKey("foo")))
-        self.failUnlessEqual(hash("foo"), hash(self.APEKey("FoO")))
-
-    def test_repr(self):
-        self.failUnless(repr(self.APEKey("foo")))
-
-class APEBinaryTest(TestCase):
-    uses_mmap = False
-
-    from mutagen.apev2 import APEBinaryValue as BV
-
-    def setUp(self):
-        self.sample = "\x12\x45\xde"
-        self.value = mutagen.apev2.APEValue(self.sample,mutagen.apev2.BINARY)
-
-    def test_type(self):
-        self.failUnless(isinstance(self.value, self.BV))
-
-    def test_const(self):
-        self.failUnlessEqual(self.sample, str(self.value))
-
-    def test_repr(self):
-        repr(self.value)
-
-class APETextTest(TestCase):
-    uses_mmap = False
-
-    from mutagen.apev2 import APETextValue as TV
-    def setUp(self):
-        self.sample = ["foo", "bar", "baz"]
-        self.value = mutagen.apev2.APEValue(
-            "\0".join(self.sample), mutagen.apev2.TEXT)
-
-    def test_type(self):
-        self.failUnless(isinstance(self.value, self.TV))
-
-    def test_list(self):
-        self.failUnlessEqual(self.sample, list(self.value))
-
-    def test_setitem_list(self):
-        self.value[2] = self.sample[2] = 'quux'
-        self.test_list()
-        self.test_getitem()
-        self.value[2] = self.sample[2] = 'baz'
-
-    def test_getitem(self):
-        for i in range(len(self.value)):
-            self.failUnlessEqual(self.sample[i], self.value[i])
-
-    def test_repr(self):
-        repr(self.value)
-
-class APEExtTest(TestCase):
-    uses_mmap = False
-
-    from mutagen.apev2 import APEExtValue as EV
-
-    def setUp(self):
-        self.sample = "http://foo"
-        self.value = mutagen.apev2.APEValue(
-            self.sample, mutagen.apev2.EXTERNAL)
-
-    def test_type(self):
-        self.failUnless(isinstance(self.value, self.EV))
-
-    def test_const(self):
-        self.failUnlessEqual(self.sample, str(self.value))
-
-    def test_repr(self):
-        repr(self.value)
-
-class TAPEv2File(TestCase):
-    uses_mmap = False
-
-    def setUp(self):
-        self.audio = APEv2File("tests/data/click.mpc")
-
-    def test_add_tags(self):
-        self.failUnless(self.audio.tags is None)
-        self.audio.add_tags()
-        self.failUnless(self.audio.tags is not None)
-        self.failUnlessRaises(ValueError, self.audio.add_tags)
-add(TAPEv2File)
+add(TAPEv2ThenID3v1Writer)
 
 class TAPEv2(TestCase):
     uses_mmap = False
 
     def setUp(self):
-        self.audio = APEv2(OLD)
+        fd, self.filename = mkstemp(".apev2")
+        os.close(fd)
+        shutil.copy(OLD, self.filename)
+        self.audio = APEv2(self.filename)
 
     def test_guess_text(self):
         from mutagen.apev2 import APETextValue
@@ -343,12 +183,161 @@ class TAPEv2(TestCase):
     def test_module_delete_empty(self):
         from mutagen.apev2 import delete
         delete(os.path.join("tests", "data", "emptyfile.mp3"))
-    
+
+    def test_invalid(self):
+        self.failUnlessRaises(IOError, mutagen.apev2.APEv2, "dne")
+
+    def test_no_tag(self):
+        self.failUnlessRaises(IOError, mutagen.apev2.APEv2,
+                              os.path.join("tests", "data", "empty.mp3"))
+
+    def test_cases(self):
+        self.failUnlessEqual(self.audio["artist"], self.audio["ARTIST"])
+        self.failUnless("artist" in self.audio)
+        self.failUnless("artisT" in self.audio)
+
+    def test_keys(self):
+        self.failUnless("Track" in self.audio.keys())
+        self.failUnless("AnArtist" in self.audio.values())
+
+        self.failUnlessEqual(
+            self.audio.items(), zip(self.audio.keys(), self.audio.values()))
+
+    def test_dictlike(self):
+        self.failUnless(self.audio.get("track"))
+        self.failUnless(self.audio.get("Track"))
+
+    def test_del(self):
+        s = self.audio["artist"]
+        del(self.audio["artist"])
+        self.failIf("artist" in self.audio)
+        self.failUnlessRaises(KeyError, self.audio.__getitem__, "artist")
+        self.audio["Artist"] = s
+        self.failUnlessEqual(self.audio["artist"], "AnArtist")
+
+    def test_values(self):
+        self.failUnlessEqual(self.audio["artist"], self.audio["artist"])
+        self.failUnless(self.audio["artist"] < self.audio["title"])
+        self.failUnlessEqual(self.audio["artist"], "AnArtist")
+        self.failUnlessEqual(self.audio["title"], "Some Music")
+        self.failUnlessEqual(self.audio["album"], "A test case")
+        self.failUnlessEqual("07", self.audio["track"])
+
+        self.failIfEqual(self.audio["album"], "A test Case")
+
+    def test_pprint(self):
+        self.failUnless(self.audio.pprint())
+
+    def tearDown(self):
+        os.unlink(self.filename)
+
 add(TAPEv2)
 
-add(APEReader)
-add(APEWriter)
-add(APEKeyTest)
-add(APEBinaryTest)
-add(APETextTest)
-add(APEExtTest)
+class TAPEv2ThenID3v1(TAPEv2):
+    uses_mmap = False
+
+    def setUp(self):
+        super(TAPEv2ThenID3v1, self).setUp()
+        f = file(self.filename, "ab+")
+        f.write("TAG" + "\x00" * 125)
+        f.close()
+        self.audio = APEv2(self.filename)
+
+add(TAPEv2ThenID3v1)
+
+class TAPEv2WithLyrics2(TestCase):
+    uses_mmap = False
+
+    def setUp(self):
+        self.tag = mutagen.apev2.APEv2(LYRICS2)
+
+    def test_values(self):
+        self.failUnlessEqual(self.tag["MP3GAIN_MINMAX"], "000,179")
+        self.failUnlessEqual(self.tag["REPLAYGAIN_TRACK_GAIN"], "-4.080000 dB")
+        self.failUnlessEqual(self.tag["REPLAYGAIN_TRACK_PEAK"], "1.008101")
+
+add(TAPEv2WithLyrics2)
+
+class TAPEBinaryValue(TestCase):
+    uses_mmap = False
+
+    from mutagen.apev2 import APEBinaryValue as BV
+
+    def setUp(self):
+        self.sample = "\x12\x45\xde"
+        self.value = mutagen.apev2.APEValue(self.sample,mutagen.apev2.BINARY)
+
+    def test_type(self):
+        self.failUnless(isinstance(self.value, self.BV))
+
+    def test_const(self):
+        self.failUnlessEqual(self.sample, str(self.value))
+
+    def test_repr(self):
+        repr(self.value)
+
+add(TAPEBinaryValue)
+
+class TAPETextValue(TestCase):
+    uses_mmap = False
+
+    from mutagen.apev2 import APETextValue as TV
+    def setUp(self):
+        self.sample = ["foo", "bar", "baz"]
+        self.value = mutagen.apev2.APEValue(
+            "\0".join(self.sample), mutagen.apev2.TEXT)
+
+    def test_type(self):
+        self.failUnless(isinstance(self.value, self.TV))
+
+    def test_list(self):
+        self.failUnlessEqual(self.sample, list(self.value))
+
+    def test_setitem_list(self):
+        self.value[2] = self.sample[2] = 'quux'
+        self.test_list()
+        self.test_getitem()
+        self.value[2] = self.sample[2] = 'baz'
+
+    def test_getitem(self):
+        for i in range(len(self.value)):
+            self.failUnlessEqual(self.sample[i], self.value[i])
+
+    def test_repr(self):
+        repr(self.value)
+
+add(TAPETextValue)
+
+class TAPEExtValue(TestCase):
+    uses_mmap = False
+
+    from mutagen.apev2 import APEExtValue as EV
+
+    def setUp(self):
+        self.sample = "http://foo"
+        self.value = mutagen.apev2.APEValue(
+            self.sample, mutagen.apev2.EXTERNAL)
+
+    def test_type(self):
+        self.failUnless(isinstance(self.value, self.EV))
+
+    def test_const(self):
+        self.failUnlessEqual(self.sample, str(self.value))
+
+    def test_repr(self):
+        repr(self.value)
+
+add(TAPEExtValue)
+
+class TAPEv2File(TestCase):
+    uses_mmap = False
+
+    def setUp(self):
+        self.audio = APEv2File("tests/data/click.mpc")
+
+    def test_add_tags(self):
+        self.failUnless(self.audio.tags is None)
+        self.audio.add_tags()
+        self.failUnless(self.audio.tags is not None)
+        self.failUnlessRaises(ValueError, self.audio.add_tags)
+add(TAPEv2File)
